@@ -1,4 +1,5 @@
-#! /usr/bin/python3
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
 # Copyright (C) 2021 Fares Ahmed
 #
 # This file is part of matrix-py.
@@ -30,13 +31,13 @@ OR $ python -m matrixpy --help
 
 # pylint: disable=C0103 # Variable name "m" is "iNvAlId-nAmE"
 
-import argparse as _argparse
-import json as _json
+import sys as _sys
 import random as _random
+import six as _six
 
 
 __all__ = ["Matrix", "MatrixError"]
-__version__ = "0.6.5"
+__version__ = "0.9.0"
 __author__ = "Fares Ahmed <faresahmed@zohomail.com>"
 
 
@@ -77,6 +78,15 @@ class Matrix:
             # list From str -> int
             self.matrix = [list(map(int, matrix[i])) for i in range(len(matrix))]
 
+        for row in self.matrix:
+            if len(row) != len(self.matrix[0]):
+                _six.raise_from(
+                    MatrixError(
+                        "Row `{}` has a different size" " from other rows".format(row)
+                    ),
+                    None,
+                )
+
         self.rowsnum = len(self.matrix)
         self.colsnum = len(self.matrix[0])
 
@@ -120,20 +130,57 @@ class Matrix:
             if len(max(row, key=len)) > maxlen:
                 maxlen = len(max(row, key=len))
 
-        for rowsidx, row in enumerate(matrix_str):
-            for numidx, num in enumerate(matrix_str[0]):
-                rows += matrix_str[rowsidx][numidx] + " "
-                rows += " " * (maxlen - len(matrix_str[rowsidx][numidx]))
+        for row in matrix_str:
+            for num in row:
+                rows += num + " "
+                rows += " " * (maxlen - len(num))
             rows = rows.rstrip()
             rows += "\n"
 
         # Calculate the spaces before (ROWSNUMxCOLSNUM)
         rwcl_spaces = " " * (
             len(rows.split("\n")[-2]) // 2
-            - len(f"({self.rowsnum}x{self.colsnum})") // 2
+            - len("({}x{})".format(self.rowsnum, self.colsnum)) // 2
         )
 
-        return rows + rwcl_spaces + f"({self.rowsnum}x{self.colsnum})"
+        return rows + rwcl_spaces + "({}x{})".format(self.rowsnum, self.colsnum)
+
+    def alignleft(self):
+        """Return the matrix in `str` representation
+
+        Appears when using print(Matrix)
+
+        >>> print(Matrix.random((3,3), 1, 1000))
+        Output: 133 23  388
+                4   335 6
+                72  8   933
+                   (3x3)
+        """
+        matrix_str = list()
+        rows = str()
+
+        for row in self.matrix:  # [[1, 2, 3]] -> [["1", "2", "3"]]
+            matrix_str.append(list(map(str, row)))
+
+        # Get the maximum number in the matrix
+        maxlen = int()
+        for row in matrix_str:
+            if len(max(row, key=len)) > maxlen:
+                maxlen = len(max(row, key=len))
+
+        for row in matrix_str:
+            for num in row:
+                rows += " " + (" " * (maxlen - len(num)) + num)
+            rows = rows.strip()
+            rows += "\n"
+
+        # Calculate the spaces before (ROWSNUMxCOLSNUM)
+        rwcl_spaces = " " * (
+            len(rows.split("\n")[-2]) // 2
+            - len("({}x{})".format(self.rowsnum, self.colsnum)) // 2
+        )
+
+        return rows.replace("\n ", "\n") + rwcl_spaces + "({}x{})".format(self.rowsnum, self.colsnum)
 
     def __getitem__(self, rowcol):
         """Return row, col, or item of MatrixObject
@@ -143,7 +190,7 @@ class Matrix:
             [Row] MatrixObject[1]  -> '4 5 6'
             [Col] MatrixObject[:1] -> '2; 5'
             [Item] MatrixObject[1, 2] -> 6
-        
+
         Note: in [Item] the first arg is the row
         num and the second is the col num.
         """
@@ -164,6 +211,29 @@ class Matrix:
             if item in row:
                 return True
         return False
+
+    def __iter__(self):
+        """for loops support
+
+        Example:
+            # to loop through all the
+            # Matrix's items:
+            for item in Matrix(3):
+                pass
+
+            # to loop through the items
+            # of a specific row in the Matrix
+            for item in Matrix(3).row(0):
+                pass
+
+            # to loop through the items
+            # of a specific col in the Matirx
+            for item in Matrix(3).col(0):
+                pass
+        """
+        for row in range(self.rowsnum):
+            for col in range(self.colsnum):
+                yield self.matrix[row][col]
     # Object Creation: END
 
     # Object Expressions: START
@@ -193,10 +263,12 @@ class Matrix:
             result = list()
 
             if self.rowsnum != other.rowsnum or self.colsnum != other.colsnum:
-                raise MatrixError(
-                    "To add matrices, the matrices must have"
-                    " the same dimensions"
-                ) from None
+                _six.raise_from(
+                    MatrixError(
+                        "To add matrices, the matrices must have" " the same dimensions"
+                    ),
+                    None,
+                )
 
             for m in range(self.rowsnum):
                 result.append([])
@@ -221,10 +293,12 @@ class Matrix:
             result = list()
 
             if self.rowsnum != other.rowsnum or self.colsnum != other.colsnum:
-                raise MatrixError(
-                    "To sub matrices, the matrices must have"
-                    " the same dimensions"
-                ) from None
+                _six.raise_from(
+                    MatrixError(
+                        "To sub matrices, the matrices must have" " the same dimensions"
+                    ),
+                    None,
+                )
 
             for m in range(self.rowsnum):
                 result.append([])
@@ -246,10 +320,13 @@ class Matrix:
         if isinstance(other, Matrix):
             # MatA * MatB
             if self.colsnum != other.rowsnum:
-                raise MatrixError(
-                    "The number of Columns in MatA must be"
-                    " equal to the number of Rows in MatB"
-                ) from None
+                _six.raise_from(
+                    MatrixError(
+                        "The number of Columns in MatA must be"
+                        " equal to the number of Rows in MatB"
+                    ),
+                    None,
+                )
 
             # References:
             # https://www.geeksforgeeks.org/python-program-multiply-two-matrices
@@ -273,7 +350,8 @@ class Matrix:
     # Object Math opertaions: END
 
     # Object Manpulation: START
-    def row(self, position: int, start=0):
+    def row(self, position, start=0):
+        # type: (int, int) -> Matrix
         """Return the row in the position `position`
 
         Using `indexing` method is recommended
@@ -296,9 +374,10 @@ class Matrix:
         try:
             return Matrix([self.matrix[position]])
         except IndexError:
-            raise MatrixError("Matrix Index out of range") from None
+            _six.raise_from(MatrixError("Matrix Index out of range"), None)
 
-    def col(self, position: int, start=0):
+    def col(self, position, start=0):
+        # type: (int, int) -> Matrix
         """Return the col in the position `position`
 
         Using `indexing` method is recommended
@@ -321,9 +400,10 @@ class Matrix:
         try:
             return Matrix([[row[position]] for row in self.matrix])
         except IndexError:
-            raise MatrixError("Matrix Index out of range") from None
+            _six.raise_from(MatrixError("Matrix Index out of range"), None)
 
     def addrow(self, row, index=-1):
+        # type: (Matrix, int) -> Matrix
         """Add a new row to your Matrix Object
         MatrixObject -> '1 2 3'
 
@@ -343,6 +423,7 @@ class Matrix:
         return Matrix(result)
 
     def addcol(self, col, index=-1):
+        # type: (Matrix, int) -> Matrix
         """Add a new col to your Matrix Object
         MatrixObject -> '1 2 3; 5 6 7'
 
@@ -364,6 +445,7 @@ class Matrix:
         return Matrix(result)
 
     def rmrow(self, index):
+        # type: (int) -> Matrix
         """Remove an Existing row from your Matrix Object.
         MatrixObject -> '1 2 3; 4 5 6'
 
@@ -378,11 +460,12 @@ class Matrix:
         try:
             result.pop(index)
         except IndexError:
-            raise MatrixError("Matrix Index out of range") from None
+            _six.raise_from(MatrixError("Matrix Index out of range"), None)
 
         return Matrix(result)
 
     def rmcol(self, index):
+        # type: (int) -> Matrix
         """Remove an Existing col from your Matrix Object.
         MatrixObject -> '1 2 3 4; 5 6 7 8'
 
@@ -398,11 +481,12 @@ class Matrix:
             for i in range(self.rowsnum):
                 result[i].pop(index)
         except IndexError:
-            raise MatrixError("Matrix Index out of range") from None
+            _six.raise_from(MatrixError("Matrix Index out of range"), None)
 
         return Matrix(result)
 
     def transpose(self):
+        # type: () -> Matrix
         """Swith the row and column indices of the Matrix
 
         Returns:
@@ -425,10 +509,11 @@ class Matrix:
         return Matrix([list(i) for i in zip(*self.matrix)])
 
     def tolist(self):
+        # type: () -> list
         """Convert Matrix Object to a Nested List
 
         Returns:
-            list: Convert Matrix Object to a 
+            list: Convert Matrix Object to a
             Nested List. Useful for manipulating
             the list yourself if matrix-py doesn't
             do what you expect.
@@ -439,10 +524,49 @@ class Matrix:
             [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
         """
         return self.matrix
+
+    # The following function (rank) is by absognety and copied from
+    # https://github.com/absognety/Competitive-Coding-Platforms/
+    # and licensed under The GPLv3 License
+    def rank(self):
+        # type() -> int
+        """Return the rank of the Matrix
+
+        Returns:
+            int: The rank of the Matrix
+        """
+        rank = self.rowsnum
+        mat = self.matrix
+
+        for row in range(rank):
+            if mat[row][row]:
+                for col in range(self.colsnum):
+                    if col != row:
+                        multiplier = mat[col][row] / mat[row][row]
+                        for i in range(rank):
+                            mat[col][i] -= multiplier * mat[row][i]
+            else:
+                reduce = True
+                for i in range(row + 1, self.colsnum):
+                    if mat[i][row]:
+                        for s in range(rank):
+                            temp = mat[row][s]
+                            mat[row][s] = mat[i][s]
+                            mat[i][s] = temp
+                        reduce = False
+                        break
+                if reduce:
+                    rank -= 1
+                    for i in range(self.colsnum):
+                        mat[i][row] = mat[i][rank]
+                row -= 1
+
+        return rank
     # Object Manpulation: END
 
     # Booleon Expressions: START
     def is_square(self):
+        # type: () -> bool
         """Return True if Matrix is square
 
         Returns:
@@ -453,12 +577,13 @@ class Matrix:
         return bool(self.rowsnum == self.colsnum)
 
     def is_symmetric(self):
+        # type: () -> bool
         """Return True if Matrix is symmetric
 
         Raises:
             MatrixError: if the given Matrix is not
             square Matrix. Explaination:
-            https://www.wikiwand.com/en/Symmetric_matrix 
+            https://www.wikiwand.com/en/Symmetric_matrix
 
         Returns:
             bool: True if the given Matrix (self)
@@ -475,7 +600,8 @@ class Matrix:
 
     # Pre Made Objects: START
     @staticmethod
-    def identity(size: int):
+    def identity(size):
+        # type: (int) -> Matrix
         """Return a new I (sizeXsize) Matrix
 
         Args:
@@ -483,8 +609,8 @@ class Matrix:
 
         Returns:
             Matrix: New I Matrix (All the diagonal
-            items == 1) 
-            
+            items == 1)
+
         Example:
             print(Matrix.identity(3)) # 3 -> (3x3) Matrix
             Output: 1 0 0
@@ -501,7 +627,8 @@ class Matrix:
         return Matrix(result)
 
     @staticmethod
-    def zero(size: int):
+    def zero(size):
+        # type: (int) -> Matrix
         """Return a new zero (sizeXsize) Matrix
 
         Args:
@@ -520,16 +647,16 @@ class Matrix:
         return Matrix([[0] * size] * size)
 
     @staticmethod
-    def diagonal(*numbers: int, fill=0):
+    def diagonal(fill=0, *numbers):
         """Get the diag of a Matrix or Generate one
 
         Returns:
             Matrix: The result depends on how you call
-            the function. 
-            
+            the function.
+
             Matrix.diagonal(MatrixObject):
             Return the diagonal of an exisiting Matrix Object.
-            
+
             Matrix.diagonal(INT, INT, ..):
             Return a new square Matrix with (INT, INT, ..) as
             the Matrix's diagonal
@@ -558,7 +685,8 @@ class Matrix:
         return Matrix(result)
 
     @staticmethod
-    def randint(size: tuple, a: int, b: int):
+    def randint(size, a, b):
+        # type: (tuple, int, int) -> Matrix
         """Return (size) Matrix with random integers in range (a, b)
 
         Args:
@@ -571,11 +699,11 @@ class Matrix:
         """
         if not isinstance(size, tuple):
             raise TypeError("arg1 `size` must be tuple. (3,3) = (3x3) Matrix")
-        if not isinstance(a, int): 
+        if not isinstance(a, int):
             raise TypeError("arg2 `a` must be int.")
-        if not isinstance(b, int): 
-            raise TypeError("arg2 `b` must be int.")
-        
+        if not isinstance(b, int):
+            raise TypeError("arg3 `b` must be int.")
+
         result = list()
         rowsnum = size[0]
         colsnum = size[1]
@@ -590,105 +718,103 @@ class Matrix:
 
 
 def _cli():
-    """The Command-Line Interface (CLI) for the module"""
+    HELP = ["help", "--help", "h", "-h"]
 
-    parser = _argparse.ArgumentParser(
-        prog="matrix-py",
-        description="matrix-py module to add, substract, multiply" "matrices.",
-        epilog='Usage: .. -ma "[[1, 2, 3], [4, 5, 6]]" -op "+" -mb'
-        ' "[[7, 8, 9], [10, 11, 12]]"',
-    )
+    OPERATROS = ["+", "-", "*"]
 
-    parser.add_argument(
-        "-v",
-        "--version",
-        action="version",
-        version=__version__,
-    )
+    args = list(map(str.lower, _sys.argv))
 
-    parser.add_argument(
-        "-s", "--size", type=_json.loads, metavar="", help="Size of MatA"
-    )
+    if len(args) < 2:
+        args.append("help")
 
-    parser.add_argument(
-        "-t",
-        "--transpose",
-        type=_json.loads,
-        metavar="",
-        help='Transpose of MatA (-t "[[1, 2, 3], [4, 5, 6]]")',
-    )
+    if args[1] in HELP or args[1].startswith("h"):
+        return """
+Welcome to matrixpy Command-Line Interface program!
 
-    parser.add_argument(
-        "-ma",
-        "--matrixa",
-        type=_json.loads,
-        metavar="",
-        help='MatA (.. -ma "[[1, 2, 3], [4, 5, 6]]")',
-    )
+\x1b[93m\x1b[1m┍————————————————————————————- /ᐠ｡ꞈ｡ᐟ\ ————————————————————————————┑
+\x1b[0m
+\x1b[33mMathmatical Operations\x1b[0m:
+\x1b[32m    Addition (+)\x1b[0m        matrixpy "1 2 3; 4 5 6" "+" "1 2 3; 4 5 6"
+\x1b[32m    Substraction (-)\x1b[0m    matrixpy "1 2 3; 4 5 6" "-" "1 2 3; 4 5 6"
+\x1b[32m    Multiplication (*)\x1b[0m  matrixpy "1 2 3; 4 5 6" "*" "1 2; 3 4; 5 6"
 
-    parser.add_argument(
-        "-op",
-        "--operator",
-        type=str,
-        metavar="",
-        help='Operator (.. -op "+", "-", "*")',
-    )
+\x1b[33mCommands\x1b[0m:
+\x1b[32m    Transpose, -t\x1b[0m       Get the transpose of a Matrix 
+                        Example: matrixpy transpose "1 2 3; 4 5 6"
 
-    parser.add_argument(
-        "-mb",
-        "--matrixb",
-        type=_json.loads,
-        metavar="",
-        help='MatB (.. -mb "[[1, 2, 3], [4, 5, 6]]")',
-    )
+\x1b[32m    Randint, -r\x1b[0m         Get a random Matrix in a specific range 
+                        Example: matrixpy randint 1 100 3x3
 
-    parser.add_argument(
-        "-I", "--identity", type=int, metavar="",
-        help="Identity (.. -I 3)"
-    )
+\x1b[33mOther\x1b[0m:
+\x1b[32m    Help, -h\x1b[0m            Get help about the CLI usage
+\x1b[32m    Version, -v\x1b[0m         Get the version of the matrixpy
 
-    parser.add_argument(
-        "-i", "--int", type=int, metavar="",
-        help="Integer (.. -i 5)"
-    )
+\x1b[93m\x1b[1m┕————————————————————————————(..)(..) ∫∫——————————————————————————-┙
+\x1b[0m"""
 
-    parser.add_argument(
-        "-diag",
-        "--diagonal",
-        type=_json.loads,
-        metavar="",
-        help="Diagonal (.. -diag [1, 2, 3, 4])",
-    )
+    if args[1].startswith("v") or args[1].startswith("-v"):
+        return __version__
 
-    args = parser.parse_args()
+    if args[1].startswith("t") or args[1].startswith("-t"):
+        try:
+            MatA = Matrix(args[2])
+        except (ValueError, IndexError, MatrixError) as e:
+            if str(e) == "list index out of range":
+                return """\x1b[31m\x1b[1mERROR\x1b[0m: Missing `MatA` argument after transpose.
+\x1b[34m\x1b[1mTIP\x1b[0m: $ matrixpy tranpose '1 2 3; 4 5 6'"""
+            return """\x1b[31m\x1b[1mERROR\x1b[0m: Please define the Matrix you
+want to transpose correctly. \x1b[34m\x1b[1mTIP\x1b[0m:
+ $ matrixpy tranpose '1 2 3; 4 5 6'"""
 
-    if args.size:
-        print(Matrix(args.size))
+        return MatA.transpose()
 
-    elif args.transpose:
-        print(Matrix(args.transpose).transpose())
+    if args[1].startswith("r") or args[1].startswith("-r"):
+        try:
+            start = int(args[2])
+            end = int(args[3])
+            matsize = args[4].split("x")
+        except (IndexError, ValueError):
+            return """\x1b[31m\x1b[1mERROR\x1b[0m: Please use the randint command correctly.
+\x1b[34m\x1b[1mTIP\x1b[0m: $ matrixpy randint 1 100 3x3"""
 
-    elif args.matrixa:
-        if args.matrixb:
-            b = Matrix(args.matrixb)
-        elif args.int:
-            b = args.int
-        elif args.identity:
-            b = Matrix.identity(args.identity)
-        elif args.diagonal:
-            b = Matrix.diagonal(args.diagonal)
+        try:
+            return Matrix.randint((int(matsize[0]), int(matsize[1])),
+                                   start,
+                                   end)
+        except ValueError:
+            return """\x1b[31m\x1b[1mERROR\x1b[0m: Please use the randint command correctly.
+\x1b[34m\x1b[1mTIP\x1b[0m: $ matrixpy randint 1 100 3x3"""
 
-        if args.operator == "+":
-            print(Matrix(args.matrixa) + b)
-        elif args.operator == "-":
-            print(Matrix(args.matrixa) - b)
-        elif args.operator == "*":
-            print(Matrix(args.matrixa) * b)
-        else:
-            raise SyntaxError("The avillable operations are +, -, *")
-    else:
-        print(parser.print_help())
+    try:
+        MatA = Matrix(args[1])
+    except (ValueError, IndexError, MatrixError):
+        return """\x1b[31m\x1b[1mERROR\x1b[0m: Please define MatA correctly.
+\x1b[34m\x1b[1mTIP\x1b[0m: $ matrixpy '1 2 3; 4 5 6'"""
+
+    try:
+        if args[2] not in OPERATROS:
+            return """\x1b[31m\x1b[1mERROR\x1b[0m: Please define a valid operator. ('+', '-', '*')
+\x1b[34m\x1b[1mTIP\x1b[0m: $ matrixpy '1 2 3; 4 5 6' '+' '1 2 3; 4 5 6'"""
+        op = args[2]
+    except IndexError:
+        return """\x1b[31m\x1b[1mERROR\x1b[0m: Please define an operator. ('+', '-', '*')
+\x1b[34m\x1b[1mTIP\x1b[0m: $ matrixpy '1 2 3; 4 5 6' '+' '1 2 3; 4 5 6'"""
+
+    try:
+        MatB = Matrix(args[3])
+    except (ValueError, IndexError, MatrixError):
+        return """\x1b[31m\x1b[1mERROR\x1b[0m: Please define MatB correctly.
+\x1b[34m\x1b[1mTIP\x1b[0m: $ matrixpy '1 2 3; 4 5 6' + '1 2 3; 4 5 6'"""
+
+    try:
+        if op == "+":
+            return MatA + MatB
+        if op == "-":
+            return MatA - MatB
+        return MatA * MatB
+    except MatrixError as MError:
+        return MError
 
 
 if __name__ == "__main__":
-    _cli()
+    print(_cli())
